__copyright__ = """
* Copyright (c) 2018 CA. All rights reserved.
*
* This software and all information contained therein is confidential and proprietary and
* shall not be duplicated, used, disclosed or disseminated in any way except as authorized
* by the applicable license agreement, without the express written permission of CA. All
* authorized reproductions must be marked with this language.
*
* EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE EXTENT
* PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE WITHOUT WARRANTY
* OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF
* MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL CA BE
* LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR
* INDIRECT, FROM THE USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST
* PROFITS, BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
* EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
"""
PRINT_PREFIX = 'CA APM PYTHON AGENT: '
# Python Stuff
PYTHONPATH = 'PYTHONPATH'
PYTHONUNBUFFERED = 'PYTHONUNBUFFERED'
PYTHONVERBOSE = 'PYTHONVERBOSE'

# Agent stuff
LOGGER_NAME = []
CA_APM_PYTHON_ACFG = 'CA_APM_PYTHON_ACFG'  # Path to Agent Config File
CA_APM_PYTHON_ICFG = 'CA_APM_PYTHON_ICFG'  # Path to Instrumentation Config File
ACFG_MTIME = 'acfg_mtime'  # ACFG file modification time
ICFG_MTIME = 'icfg_mtime'  # ICFG file modification time
CA_APM_PYTHON_HOST_NAME = 'host'         # Hostname to connect to Collector Agent/ APMIA Package
CA_APM_PYTHON_PORT_NUM = 'port'          # Port number required to connect to Collector Agent/ APMIA Package
CA_APM_PYTHON_APP_NAME = 'app_name'      # Application Name to be displayed in EM
