__copyright__ = """
* Copyright (c) 2018 CA. All rights reserved.
*
* This software and all information contained therein is confidential and proprietary and
* shall not be duplicated, used, disclosed or disseminated in any way except as authorized
* by the applicable license agreement, without the express written permission of CA. All
* authorized reproductions must be marked with this language.
*
* EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE EXTENT
* PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE WITHOUT WARRANTY
* OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF
* MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL CA BE
* LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR
* INDIRECT, FROM THE USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST
* PROFITS, BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
* EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
"""
OP = 'op'
PROBE = 'probe'
COLLECTOR = 'collector'
VER = 'ver'
VERSION = 'version'
INSTID = 'instid'
PGM = 'pgm'
CMD = 'cmd'
MOD = 'mod'
PRMS = 'prms'
QUERY = 'query'
URL = 'url'
PID = 'pid'
TID = 'tid'
TIMESTAMP = 'ts'
SEQ = 'seq'
CSEQ = 'cseq'
FUNCTION_NAME = 'fn'
FUNCTION_ID = 'fnid'
METRIC_ID = 'mid'
METRIC_TYPE = 'mtype'
VALUE = 'val'
COUNT = 'ct'
MESSAGE = 'msg'
EXCEPTION = 'exc'
CLASS = 'class'
FILE = 'file'
LINE = 'line'
TRACE_ID = 'traceId'
CORR_ID = 'corId'
HTTP_CORID = 'HTTP_CORID'
RESPONSE_CODE = 'resCode'
RESPONSE_MESSAGE = 'resMsg'
HASH_SEPARATED = '<0..n>'
SKIP_INSTRUMENT = '<0:..>'
SPEAK = 'speak'
PAUSE = 'pause'
CONTINUE = 'continue'
CONFIG = 'config'
MAP = 'map'
REQUIRE = 'require'
COMMAND_CONN = 'commandConn'
DATA_CONN = 'dataConn'
ARF = 'arf'
FUNCTION_CALL = 'fnC'
FUNCTION_RET = 'fnR'
ARF_VER = 1
METRIC = 'm'
LOG = 'log'
PYTHON = 'python'
CONTAINER_ID = 'containerId'
PYTHON_MODULE = 'Python Module'
PYTHON_RASA_INTENT = 'Python Rasa Intent'
CORIDTEMPLATE = 'AgAAASVIQgAAAAFGAAAAAQAAABFqYXZhLnV0aWwuSGFzaE1hcAAAAARIQgAAAAJGAAAAAgAAABBqYXZhLmxhbmcuU3RyaW5nAApUeG5UcmFjZUlkSEIAAAADRQAAAAIAITZCOThGQURFMEE4NDk4MjU1MzI3MjFGRDhFNEE2M0RGM0hCAAAABEUAAAACABNDYWxsZXIgQ29tcG9uZW50IElESEIAAAAFRQAAAAIAATFIQgAAAAZFAAAAAgAPQ2FsbGVyVGltZXN0YW1wSEIAAAAHRQAAAAIADTE1NjA4NzgzNzE5NjFIQgAAAAhFAAAAAgARVXBzdHJlYW1HVUlEQ2FjaGVIQgAAAAlGAAAAAwAAABNqYXZhLnV0aWwuQXJyYXlMaXN0AAAAAA=='
